'use strict';

/**
 * Traversal listener to identify i18n function calls, retrieve the string value and add mount translation files automatically.
 */

var fs = require('fs');
var path = require('path');

// The generated output file format. For now, just a regular JSON file.
var SUPPORTED_FILE_FORMAT = '.json';

// Babel plugin that receives the babel instance as the main argument. Has the Abstract Syntax Tree (AST) and options.
var I18nPlugin = function I18nPlugin(babel) {
    // Should store the current execution's strings
    var cache = new Map();

    // Keep the current set of strings to be translated
    var translationSet = new Map();

    // Name of the function, can be changed by plugin's options
    var functionAlias;
    // Languages to map the strigs to
    var languageSet;
    // Message to be included in the translation's files
    var todoMessage;

    var t = babel.types;
    // Input path for the existent files to be loaded
    var languageInputPath;
    // Output path for the files to be written
    var languageOutputPath;

    var resolvePath = function resolvePath(fileName) {
        return path.join(__dirname, fileName);
    };

    // Pre function that runs before the plugins
    var pre = function pre(state) {
        // Plugin has path and config, we just want to retrieve the config
        var plugin = state.opts.plugins.filter(function (plugin) {
            return plugin[0].key === 'babel-plugin-dynamic-i18n';
        });

        var _ref = plugin && plugin[0][1] || {},
            alias = _ref.alias,
            todoTag = _ref.todoTag,
            languages = _ref.languages,
            inputPath = _ref.inputPath,
            outputPath = _ref.outputPath;

        functionAlias = alias || '__';
        languageSet = languages || [];
        languageInputPath = inputPath || undefined;
        languageOutputPath = outputPath || undefined;
        todoMessage = ''.concat(todoTag || '// TODO');

        // Read the files here or create them and include the keys inside the map
        languageSet.forEach(function (language) {
            // Include a new entry for the language in cache in the map
            cache.set(language, {});

            try {
                var fileLocation = process.cwd() + '/' + languageInputPath + '/' + language + '.json';
                var fileContent = JSON.parse(fs.readFileSync(fileLocation));
                // Append file content to the language's map.
                cache.set(language, fileContent);
            } catch (error) {
                // No file was found but that's ok. We don't want to create new files now.
            }
        });
    };

    // Post function that runs after the plugins
    var post = function post(state) {
        languageSet.forEach(function (language) {
            // Remove from cached values all the keys not present in the current translation set.
            Object.keys(cache.get(language)).forEach(function (value, key) {
                var translationString = translationSet.get(key);
                if (!translationString) {
                    delete cache.get(language)[key];
                }
            });

            // Iterate over the translationSet, filter by operation add and include in cache
            translationSet.forEach(function (value, key) {
                if (value.operation && value.operation === 'add') {
                    // Create a new translation entry
                    var translationEntry = {};
                    translationEntry[key] = value.translationValue;

                    var newLanguageSet = Object.assign({}, cache.get(language), translationEntry);
                    cache.set(language, newLanguageSet);
                }
            });

            // Finally, create a new JSON file for translation, according to the cached map
            try {
                var outputLocation = process.cwd() + '/' + languageOutputPath + '/' + language + '.json';
                fs.writeFileSync(outputLocation, JSON.stringify(cache.get(language), null, 4));
            } catch (err) {
                console.log(err);
            }
        });
    };

    /**
     * Check if the current node is the translation one, according to the operator.
     * Example: var something = __('Something')
     * Operator Alias: __ (can be defined by plugins params)
     * The AST will keep a node for the call of the expression __('') and the callee name will be __
     */
    var isTranslationNode = function isTranslationNode(node, operator) {
        return node && node.callee.name === operator;
    };

    return {
        pre: pre,
        // The traversal 'walk' visiting method, will visit all nodes but we only need to listen to the CallExpression node (for the operator alias)
        visitor: {
            CallExpression: function CallExpression(path) {
                var node = path.node;

                if (isTranslationNode(node, functionAlias)) {
                    var translationKey = node.arguments[0].value;
                    var translationValue = translationKey.concat(todoMessage);
                    // Check if first language file does not have the current string key and include it. Otherwise, just include the key to be checked
                    if (cache.get(languageSet[0])[translationKey]) {
                        translationSet.set(translationKey, { translationValue: translationValue, operation: '' });
                    } else {
                        translationSet.set(translationKey, { translationValue: translationValue, operation: 'add' });
                    }
                }
            }
        },
        post: post
    };
};

module.exports = I18nPlugin;